/*=============================================================================
	GalaxySubsystem.h: Reconstructed UGalaxyAudioSubsystem.
=============================================================================*/

/*------------------------------------------------------------------------------
	UGalaxyAudioSubsystem definition.
------------------------------------------------------------------------------*/

#ifndef NAMES_ONLY

//
// Information about a playing sound.
//
class FPlayingSound
{
public:
	glxVoice*		Channel;// Probably legit.
	AActor*		Actor;		// Verified.
	INT			Id;					// Seems legit.
	UBOOL		Is3D;				// Seems legit.
	USound*		Sound;		// Verified.
	FVector		Location;	// Verified.
	FLOAT		Volume;			// Verified.
	FLOAT		Radius;			// Seems legit.
	FLOAT		Pitch;			// Verified.
	FLOAT		Priority;		// Seems legit.
	FPlayingSound()
	:	Channel	(NULL)
	,	Actor	(NULL)
	,	Id		(0)
	,	Is3D	(0)
	,	Sound	(0)
	,	Priority(0)
	,	Volume	(0)
	,	Radius	(0)
	,	Pitch	(0)
	{}
	FPlayingSound( AActor* InActor, INT InId, USound* InSound, FVector InLocation, FLOAT InVolume, FLOAT InRadius, FLOAT InPitch, FLOAT InPriority )
	:	Channel	(NULL)
	,	Actor	(InActor)
	,	Id		(InId)
	,	Is3D    (0)
	,	Sound	(InSound)
	,	Location(InLocation)
	,	Volume	(InVolume)
	,	Radius	(InRadius)
	,	Pitch	(InPitch)
	,	Priority(InPriority)
	{}
};

#define MAX_EFFECTS_CHANNELS 32

//
// The implementation of UGalaxyAudioSubsystem.
//
class GALAXY_API UGalaxyAudioSubsystem : public UAudioSubsystem
{
	DECLARE_CLASS(UGalaxyAudioSubsystem,UAudioSubsystem,CLASS_Config|CLASS_Transient)

	// Configuration.
	UBOOL UseDirectSound;
	UBOOL UseFilter;
	UBOOL UseSurround;
	UBOOL UseStereo;
	UBOOL UseCDMusic;
	UBOOL UseDigitalMusic;
	UBOOL UseReverb;
	UBOOL Use3dHardware;
	UBOOL ReverseStereo;
	UBOOL LowSoundQuality;
	UBOOL Initialized;
	FLOAT AmbientFactor;
	FLOAT DopplerSpeed;
	INT Latency;
	INT EffectsChannels;
	BYTE OutputRate;
	BYTE MusicVolume;
	BYTE SoundVolume;
	FLOAT TrackSeconds[10];

	// Unknown.
	INT _94h;
	void* _98h; // Looks like a pointer
	FLOAT _9Ch; // Gets set to zero after glxStartCDAudio() / Some time, gets update if _0BCh is 0
	INT _0A0h;  // Gets set to zero after glxStartCDAudio()
	UPlayer* _0A4h; // Looks like sth. CD Track related, but seems to have Actor on the same spot as UPlayer
	INT _0A8h;
	
	// Variables.
	FLOAT TracksTime;
	void* _0B0h; // Looks like a pointer
	INT _0B4h;
	INT _0B8h;	 // if ( _0B8h ) { CDInit(); } in Update()
	UBOOL _0BCh; // Looks like a bool, seems always 1
	BYTE StartCDTrack;
	BYTE CDTracks;
	UBOOL Using3dHardware;
	UBOOL UsingAurealHardware;
	UViewport* Viewport;
	FPlayingSound PlayingSounds[MAX_EFFECTS_CHANNELS];
	FLOAT LastTime;
	INT _6D4h; // Looks like a Tick() counter
	UMusic* CurrentMusic;
	BYTE CurrentCDTrack;
	BYTE CurrentSection;
	TCHAR _6DEh[54];
	INT _74Ch;
	FLOAT MusicFade;

	// Constructor.
	UGalaxyAudioSubsystem();
	void StaticConstructor();

	// UObject interface.
	void Destroy();
	void ShutdownAfterError();
	void PostEditChange();

	// FExec interface.
	UBOOL Exec( const TCHAR* Cmd, FOutputDevice& Ar=*GLog );

	// USubsystem interface.

	// UAudioSubsystem interface.
	UBOOL Init();
	void SetViewport( UViewport* Viewport );
	void Update( FPointRegion Region, FCoords& Coords );
	void RegisterMusic( UMusic* Music );
	void RegisterSound( USound* Sound );
	void UnregisterSound( USound* Sound );
	void UnregisterMusic( UMusic* Music );
	UBOOL PlaySoundW( AActor* Actor, INT Id, USound* Sound, FVector Location, FLOAT Volume, FLOAT Radius, FLOAT Pitch );
	void NoteDestroy( AActor* Actor );
	UBOOL GetLowQualitySetting();
	UViewport* GetViewport();
	void RenderAudioGeometry( FSceneNode* Frame );

	// Internal functions.
	void SetVolumes();
	void StopSound( INT Index );
	FLOAT SoundPriority( UViewport* Viewport, FVector Location, FLOAT Volume, FLOAT Radius );
	glxSample* UGalaxyAudioSubsystem::GetSound( USound* Sound );
	void CDInit();
};

#endif

/*-----------------------------------------------------------------------------
	The End.
-----------------------------------------------------------------------------*/
